package malaria.relogo

import static repast.simphony.relogo.Utility.*;
import static repast.simphony.relogo.UtilityG.*;
import repast.simphony.random.RandomHelper
import repast.simphony.relogo.Plural;
import repast.simphony.relogo.Stop;
import repast.simphony.relogo.Utility;
import repast.simphony.relogo.UtilityG;
import repast.simphony.relogo.schedule.Go;
import repast.simphony.relogo.schedule.Setup;
import malaria.ReLogoTurtle;

@Plural("Mosquitoes")
class Mosquito extends ReLogoTurtle {
	def infected = false
	def infectionStage = 0
	def newlyInfected = false
	def infectedColor = orange()
	def susceptibleColor = blue()
	
	def init() {
		if(RandomHelper.nextDouble() <= mosquitoInitialInfectionProbability) {
			becomeInfected()
			infectionStage = 1 + random(mosquitoInfectionPeriod)
		}
		else {
			becomeSusceptible()
		}
	}
	
	def act() {
		move()
		feed()
	}
	
	def postStepUpdates() {
		updateInfectionStage()
		activateNewInfection()
	}
	
	def becomeNewlyInfected() {
		newlyInfected = true
	}

	private def move() {
		def distance = RandomHelper.nextDoubleFromTo(0, mosquitoMaximumMovement)
		left(random(360))
		forward(distance)
	}

	private def feed() {
		def victims = inRadius(humans(), mosquitoInfectionRadius)
	    def catchInfection = false
		if(infected) {
			ask(victims) {
				if(infected) {
					catchInfection = true
				}
				becomeNewlyInfected()
			}
		}
		else {
			// See if the mosquito is infected by one of its human neighbors.
			def infectedHumans = victims.collect() { 
				infected
		    }
			catchInfection = !infectedHumans.isEmpty()
		}
		// Did this mosquito catch an infection from one of its human neighbors.
		if(catchInfection) {
			becomeNewlyInfected()
		}
	}

	private def updateInfectionStage() {
		if(infected) {
			infectionStage++
			if(infectionStage > mosquitoInfectionPeriod) {
				becomeSusceptible()
			}
		}
	}
	
	private def activateNewInfection() {
		if(newlyInfected) {
			becomeInfected()
			newlyInfected = false
		}
	}

	private def becomeInfected() {
		infected = true
		infectionStage = 0
		setColor(infectedColor)
	}

	private def becomeSusceptible() {
		if(infected) {
			infected = false
			infectionStage = 0
		}
		setColor(susceptibleColor)
	}
	
	def infectionCount() {
		infected ? 1 : 0
	}
}
