package simulation;

/**
 * An implementation of Gillespie's Direct Method SSA with the addition of delays.
 * Gillespie, D.T.: A general method for numerically simulating the stochastic time evolution
 *  of coupled chemical reactions. Journal of Computational Physics 22(403) (1976)
 * Gillespie, D.T.: Exact stochastic simulation of coupled chemical reactions.
 * Journal of Physical Chemistry 81(25), 23402361 (1977)
 * Bratsun, D., Volfson, D., Tsimring, L.S., Hasty, J.: Delay-induced stochastic oscillations
 * in gene regulation. Proceedings of the National Academy of Sciences of the United States of
 * America 102(41), 14,59314,598 (2005)
 * Roussel, M.R., Zhu, R.: Validation of an algorithm for delay stochastic simulation of 
 * transcription and translation in prokaryotic gene expression. Physical Biology 3, 274284 (2006)
 * 
 * @author djb
 *
 */
public class Main 
{
	/**
	 * Program starting point.
	 * Stopping time and output sampling rate may be specified via
	 * program parameters.
	 * @param args Optional stop time and output sampling rate in steps.
	 */
	public static void main(String[] args)
	{
		double stopTime = 200;
		int samplingRate = 1;
		if(args.length > 0) {
			stopTime = Double.parseDouble(args[0]);
			if(args.length > 1) {
				samplingRate = Integer.parseInt(args[1]);
			}
		}
		Model model = new Model(stopTime, samplingRate);
		model.run();
	}
}
