package simulation;

/**
 * A product in the queue of pending products.
 * It is produced at a given production time.
 * @author djb
 *
 */
public class PendingProduct extends Product implements Comparable<PendingProduct>
{
	// When the product is produced.
	private double productionTime;
	
	/**
	 * Create a PendingProduct
	 * @param molecule The product.
	 * @param multiplicity Multiplicity of the product.
	 * @param productionTime The production time.
	 */
	public PendingProduct(Molecule molecule, int multiplicity, double productionTime)
	{
		super(molecule, multiplicity);
		this.productionTime = productionTime;
	}

	/**
	 * Return the production time.
	 * @return The production time.
	 */
	public double getProductionTime()
	{
		return productionTime;
	}

	/**
	 * Compare this pending product with another.
	 * @return An approximation of the distance between this product and the other.
	 */
	public int compareTo(PendingProduct other)
	{
		// Scale to convert difference to a meaningful int value.
		return (int) ((productionTime - other.productionTime) * 10000);
	}
}
