package simulation;

/**
 * An implementation of Gillespie's Direct Method SSA.
 * Gillespie, D.T.: A general method for numerically simulating the stochastic time evolution
 * of coupled chemical reactions. Journal of Computational Physics 22(403) (1976)
 * Gillespie, D.T.: Exact stochastic simulation of coupled chemical reactions.
 * Journal of Physical Chemistry 81(25), 2340-2361 (1977)
 * 
 * @author David J. Barnes (d.j.barnes@kent.ac.uk)
 *
 */
public class Main 
{
	/**
	 * Program starting point.
	 * Stopping time and output sampling rate may be specified via
	 * program parameters.
	 * @param args Optional stop time and output sampling rate in steps.
	 */
	public static void main(String[] args)
	{
		double stopTime = 200;
		int samplingRate = 1;
		if(args.length > 0) {
			stopTime = Double.parseDouble(args[0]);
			if(args.length > 1) {
				samplingRate = Integer.parseInt(args[1]);
			}
		}
		Model model = new Model(stopTime, samplingRate);
		model.run();
	}
}
