package simulation;

/**
 * An implementation of Slepoy, Thompson and Plimpton's SSA-CR variation
 * of Gillespie's SSA.
 * 
 * Slepoy, A., Thompson, A.P., Plimpton, S.J.: A constant-time kinetic
 * Monte Carlo algorithm for simulation of large biochemical reaction networks.
 * Journal of Chemical Physics 128 (2008)
 * 
 * @author David J. Barnes (d.j.barnes@kent.ac.uk)
 *
 */
public class Main 
{
	/**
	 * Program starting point.
	 * Stopping time and output sampling rate may be specified via
	 * program parameters.
	 * @param args Optional stop time and output sampling rate in steps.
	 */
	public static void main(String[] args)
	{
		double stopTime = 200;
		int samplingRate = 10;
		if(args.length > 0) {
			stopTime = Double.parseDouble(args[0]);
			if(args.length > 1) {
				samplingRate = Integer.parseInt(args[1]);
			}
		}
		Model model = new Model(stopTime, samplingRate);
		model.run();
	}
}
