package simulation;

/**
 * Model a reaction product - a molecule with a multiplicity.
 *
 * @author David J. Barnes (d.j.barnes@kent.ac.uk)
 *
 */
public class Product
{
	// The molecule.
	private Molecule molecule;
	// The molecule's multiplicity within the reaction.
	private int multiplicity;
	
	/**
	 * Create a product.
	 * @param molecule The product molecule.
	 * @param multiplicity The molecule's multiplicity in the reaction.
	 */
	public Product(Molecule molecule, int multiplicity)
	{
		this.molecule = molecule;
		this.multiplicity = multiplicity;
	}

	/**
	 * Produce this product.
	 */
	public void produce()
	{
		molecule.delta(multiplicity);
	}

	/**
	 * Return the molecule associated with this product.
	 * @return The molecule associated with this product.
	 */
	public Molecule getMolecule()
	{
		return molecule;
	}

	/**
	 * Return the multiplicity of the molecule.
	 * @return The multiplicity of the molecule.
	 */
	public int getMultiplicity()
	{
		return multiplicity;
	}

	/**
	 * Return a string representation of the product.
	 * @return A string representation of the product.
	 */
	public String toString()
	{
		StringBuilder builder = new StringBuilder();
		if(multiplicity > 1) {
			builder.append(multiplicity);
			builder.append('.');
		}
		builder.append(molecule.getName());
		return builder.toString();	
	}
}
