package simulation;

/**
 * Model a reactant - a molecule with a multiplicity within
 * the reaction.
 * 
 * @author David J. Barnes (d.j.barnes@kent.ac.uk)
 *
 */
public class Reactant 
{
	// The molecule in the reaction.
	private Molecule molecule;
	// The multiplicity of the molecule in the reaction.
	private int multiplicity;
	
	/**
	 * Create a reactant. 
	 * @param molecule The reactant molecule.
	 * @param multiplicity The molecule's multiplicity in the reaction.
	 */
	public Reactant(Molecule molecule, int multiplicity)
	{
		this.molecule = molecule;
		this.multiplicity = multiplicity;
	}
	
	/**
	 * Return the multiplicity of the molecule.
	 * @return The multiplicity of the molecule.
	 */
	public int getMultiplicity()
	{
		return multiplicity;
	}

	/**
	 * Return the molecule associated with this reactant.
	 * @return The molecule associated with this reactant.
	 */
	public Molecule getMolecule()
	{
		return molecule;
	}

	/**
	 * Consume this reactant in a reaction.
	 */
	public void consume()
	{
		molecule.delta(-multiplicity);
	}

	/**
	 * Return whether this reactant is consumed by the reaction or not.
	 * @return true if this is consumed, false otherwise.
	 */
	public boolean isConsumed()
	{
		return !molecule.isBoundarySpecies();
	}
	
	/**
	 * Return a string representation of the reactant.
	 * @return A string representation of the reactant.
	 */
	public String toString()
	{
		StringBuilder builder = new StringBuilder();
		if(multiplicity > 1) {
			builder.append(multiplicity);
			builder.append('.');
		}
		builder.append(molecule.getName());
		return builder.toString();	
	}
}
